# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.AndroidEnterprise.V1.Model.Install do
  @moduledoc """
  The existence of an Installs resource indicates that an app is installed on a particular device (or that an install is pending). The API can be used to create an install resource using the update method. This triggers the actual install of the app on the device. If the user does not already have an entitlement for the app, then an attempt is made to create one. If this fails (for example, because the app is not free and there is no available license), then the creation of the install fails. The API can also be used to update an installed app. If the update method is used on an existing install, then the app will be updated to the latest available version. Note that it is not possible to force the installation of a specific version of an app: the version code is read-only. If a user installs an app themselves (as permitted by the enterprise), then again an install resource and possibly an entitlement resource are automatically created. The API can also be used to delete an install resource, which triggers the removal of the app from the device. Note that deleting an install does not automatically remove the corresponding entitlement, even if there are no remaining installs. The install resource will also be deleted if the user uninstalls the app themselves.

  ## Attributes

  *   `installState` (*type:* `String.t`, *default:* `nil`) - Install state. The state "installPending" means that an install request has recently been made and download to the device is in progress. The state "installed" means that the app has been installed. This field is read-only.
  *   `productId` (*type:* `String.t`, *default:* `nil`) - The ID of the product that the install is for. For example, "app:com.google.android.gm".
  *   `versionCode` (*type:* `integer()`, *default:* `nil`) - The version of the installed product. Guaranteed to be set only if the install state is "installed".
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :installState => String.t() | nil,
          :productId => String.t() | nil,
          :versionCode => integer() | nil
        }

  field(:installState)
  field(:productId)
  field(:versionCode)
end

defimpl Poison.Decoder, for: GoogleApi.AndroidEnterprise.V1.Model.Install do
  def decode(value, options) do
    GoogleApi.AndroidEnterprise.V1.Model.Install.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.AndroidEnterprise.V1.Model.Install do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
