# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.AndroidEnterprise.V1.Model.Product do
  @moduledoc """
  A Products resource represents an app in the Google Play store that is available to at least some users in the enterprise. (Some apps are restricted to a single enterprise, and no information about them is made available outside that enterprise.) The information provided for each product (localized name, icon, link to the full Google Play details page) is intended to allow a basic representation of the product within an EMM user interface.

  ## Attributes

  *   `appRestrictionsSchema` (*type:* `GoogleApi.AndroidEnterprise.V1.Model.AppRestrictionsSchema.t`, *default:* `nil`) - The app restriction schema
  *   `appTracks` (*type:* `list(GoogleApi.AndroidEnterprise.V1.Model.TrackInfo.t)`, *default:* `nil`) - The tracks visible to the enterprise.
  *   `appVersion` (*type:* `list(GoogleApi.AndroidEnterprise.V1.Model.AppVersion.t)`, *default:* `nil`) - App versions currently available for this product.
  *   `authorName` (*type:* `String.t`, *default:* `nil`) - The name of the author of the product (for example, the app developer).
  *   `availableCountries` (*type:* `list(String.t)`, *default:* `nil`) - The countries which this app is available in.
  *   `availableTracks` (*type:* `list(String.t)`, *default:* `nil`) - Deprecated, use appTracks instead.
  *   `category` (*type:* `String.t`, *default:* `nil`) - The app category (e.g. RACING, SOCIAL, etc.)
  *   `contentRating` (*type:* `String.t`, *default:* `nil`) - The content rating for this app.
  *   `description` (*type:* `String.t`, *default:* `nil`) - The localized promotional description, if available.
  *   `detailsUrl` (*type:* `String.t`, *default:* `nil`) - A link to the (consumer) Google Play details page for the product.
  *   `distributionChannel` (*type:* `String.t`, *default:* `nil`) - How and to whom the package is made available. The value publicGoogleHosted means that the package is available through the Play store and not restricted to a specific enterprise. The value privateGoogleHosted means that the package is a private app (restricted to an enterprise) but hosted by Google. The value privateSelfHosted means that the package is a private app (restricted to an enterprise) and is privately hosted.
  *   `features` (*type:* `list(String.t)`, *default:* `nil`) - Noteworthy features (if any) of this product.
  *   `fullDescription` (*type:* `String.t`, *default:* `nil`) - The localized full app store description, if available.
  *   `iconUrl` (*type:* `String.t`, *default:* `nil`) - A link to an image that can be used as an icon for the product. This image is suitable for use at up to 512px x 512px.
  *   `lastUpdatedTimestampMillis` (*type:* `String.t`, *default:* `nil`) - The approximate time (within 7 days) the app was last published, expressed in milliseconds since epoch.
  *   `minAndroidSdkVersion` (*type:* `integer()`, *default:* `nil`) - The minimum Android SDK necessary to run the app.
  *   `permissions` (*type:* `list(GoogleApi.AndroidEnterprise.V1.Model.ProductPermission.t)`, *default:* `nil`) - A list of permissions required by the app.
  *   `productId` (*type:* `String.t`, *default:* `nil`) - A string of the form *app:<package name>*. For example, app:com.google.android.gm represents the Gmail app.
  *   `productPricing` (*type:* `String.t`, *default:* `nil`) - Whether this product is free, free with in-app purchases, or paid. If the pricing is unknown, this means the product is not generally available anymore (even though it might still be available to people who own it).
  *   `recentChanges` (*type:* `String.t`, *default:* `nil`) - A description of the recent changes made to the app.
  *   `requiresContainerApp` (*type:* `boolean()`, *default:* `nil`) - Deprecated.
  *   `screenshotUrls` (*type:* `list(String.t)`, *default:* `nil`) - A list of screenshot links representing the app.
  *   `signingCertificate` (*type:* `GoogleApi.AndroidEnterprise.V1.Model.ProductSigningCertificate.t`, *default:* `nil`) - The certificate used to sign this product.
  *   `smallIconUrl` (*type:* `String.t`, *default:* `nil`) - A link to a smaller image that can be used as an icon for the product. This image is suitable for use at up to 128px x 128px.
  *   `title` (*type:* `String.t`, *default:* `nil`) - The name of the product.
  *   `workDetailsUrl` (*type:* `String.t`, *default:* `nil`) - A link to the managed Google Play details page for the product, for use by an Enterprise admin.
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :appRestrictionsSchema =>
            GoogleApi.AndroidEnterprise.V1.Model.AppRestrictionsSchema.t() | nil,
          :appTracks => list(GoogleApi.AndroidEnterprise.V1.Model.TrackInfo.t()) | nil,
          :appVersion => list(GoogleApi.AndroidEnterprise.V1.Model.AppVersion.t()) | nil,
          :authorName => String.t() | nil,
          :availableCountries => list(String.t()) | nil,
          :availableTracks => list(String.t()) | nil,
          :category => String.t() | nil,
          :contentRating => String.t() | nil,
          :description => String.t() | nil,
          :detailsUrl => String.t() | nil,
          :distributionChannel => String.t() | nil,
          :features => list(String.t()) | nil,
          :fullDescription => String.t() | nil,
          :iconUrl => String.t() | nil,
          :lastUpdatedTimestampMillis => String.t() | nil,
          :minAndroidSdkVersion => integer() | nil,
          :permissions => list(GoogleApi.AndroidEnterprise.V1.Model.ProductPermission.t()) | nil,
          :productId => String.t() | nil,
          :productPricing => String.t() | nil,
          :recentChanges => String.t() | nil,
          :requiresContainerApp => boolean() | nil,
          :screenshotUrls => list(String.t()) | nil,
          :signingCertificate =>
            GoogleApi.AndroidEnterprise.V1.Model.ProductSigningCertificate.t() | nil,
          :smallIconUrl => String.t() | nil,
          :title => String.t() | nil,
          :workDetailsUrl => String.t() | nil
        }

  field(:appRestrictionsSchema, as: GoogleApi.AndroidEnterprise.V1.Model.AppRestrictionsSchema)
  field(:appTracks, as: GoogleApi.AndroidEnterprise.V1.Model.TrackInfo, type: :list)
  field(:appVersion, as: GoogleApi.AndroidEnterprise.V1.Model.AppVersion, type: :list)
  field(:authorName)
  field(:availableCountries, type: :list)
  field(:availableTracks, type: :list)
  field(:category)
  field(:contentRating)
  field(:description)
  field(:detailsUrl)
  field(:distributionChannel)
  field(:features, type: :list)
  field(:fullDescription)
  field(:iconUrl)
  field(:lastUpdatedTimestampMillis)
  field(:minAndroidSdkVersion)
  field(:permissions, as: GoogleApi.AndroidEnterprise.V1.Model.ProductPermission, type: :list)
  field(:productId)
  field(:productPricing)
  field(:recentChanges)
  field(:requiresContainerApp)
  field(:screenshotUrls, type: :list)
  field(:signingCertificate, as: GoogleApi.AndroidEnterprise.V1.Model.ProductSigningCertificate)
  field(:smallIconUrl)
  field(:title)
  field(:workDetailsUrl)
end

defimpl Poison.Decoder, for: GoogleApi.AndroidEnterprise.V1.Model.Product do
  def decode(value, options) do
    GoogleApi.AndroidEnterprise.V1.Model.Product.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.AndroidEnterprise.V1.Model.Product do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
