# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.AndroidEnterprise.V1.Model.WebApp do
  @moduledoc """
  A WebApps resource represents a web app created for an enterprise. Web apps are published to managed Google Play and can be distributed like other Android apps. On a user's device, a web app opens its specified URL.

  ## Attributes

  *   `displayMode` (*type:* `String.t`, *default:* `nil`) - The display mode of the web app. Possible values include: - "minimalUi", the device's status bar, navigation bar, the app's URL, and a refresh button are visible when the app is open. For HTTP URLs, you can only select this option. - "standalone", the device's status bar and navigation bar are visible when the app is open. - "fullScreen", the app opens in full screen mode, hiding the device's status and navigation bars. All browser UI elements, page URL, system status bar and back button are not visible, and the web app takes up the entirety of the available display area. 
  *   `icons` (*type:* `list(GoogleApi.AndroidEnterprise.V1.Model.WebAppIcon.t)`, *default:* `nil`) - A list of icons representing this website. If absent, a default icon (for create) or the current icon (for update) will be used.
  *   `isPublished` (*type:* `boolean()`, *default:* `nil`) - A flag whether the app has been published to the Play store yet.
  *   `startUrl` (*type:* `String.t`, *default:* `nil`) - The start URL, i.e. the URL that should load when the user opens the application.
  *   `title` (*type:* `String.t`, *default:* `nil`) - The title of the web app as displayed to the user (e.g., amongst a list of other applications, or as a label for an icon).
  *   `versionCode` (*type:* `String.t`, *default:* `nil`) - The current version of the app. Note that the version can automatically increase during the lifetime of the web app, while Google does internal housekeeping to keep the web app up-to-date.
  *   `webAppId` (*type:* `String.t`, *default:* `nil`) - The ID of the application. A string of the form "app:<package name>" where the package name always starts with the prefix "com.google.enterprise.webapp." followed by a random id.
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :displayMode => String.t() | nil,
          :icons => list(GoogleApi.AndroidEnterprise.V1.Model.WebAppIcon.t()) | nil,
          :isPublished => boolean() | nil,
          :startUrl => String.t() | nil,
          :title => String.t() | nil,
          :versionCode => String.t() | nil,
          :webAppId => String.t() | nil
        }

  field(:displayMode)
  field(:icons, as: GoogleApi.AndroidEnterprise.V1.Model.WebAppIcon, type: :list)
  field(:isPublished)
  field(:startUrl)
  field(:title)
  field(:versionCode)
  field(:webAppId)
end

defimpl Poison.Decoder, for: GoogleApi.AndroidEnterprise.V1.Model.WebApp do
  def decode(value, options) do
    GoogleApi.AndroidEnterprise.V1.Model.WebApp.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.AndroidEnterprise.V1.Model.WebApp do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
